@@ -1,13 +1,20 @@
-from graphgallery.utils.conversion import (chk_convert, sparse_adj_to_edges,
+from graphgallery.utils.conversion import (check_and_convert, sparse_adj_to_edges,
                                            sparse_tensor_to_sparse_adj,
                                            sparse_adj_to_edges,
                                            edges_to_sparse_adj,
                                            asintarr,
                                            astensor)
+
 from graphgallery.utils.data_utils import normalize_adj, normalize_x, Bunch, sample_mask
 from graphgallery.utils.shape_utils import repeat
+from graphgallery.utils.tqdm import tqdm
 
 
-__version__ = '0.1.2'
+from graphgallery import nn
+from graphgallery import utils
+from graphgallery import sequence
 
-__all__ = ['graphgallery', '__version__']
\ No newline at end of file
+
+__version__ = '0.1.5'
+
+__all__ = ['graphgallery', 'nn', 'utils', 'sequence', '__version__']
